#ifndef IO_NODES_GPIO_H
#define IO_NODES_GPIO_H IO_NODES_GPIO_H

#include <cstddef>
#include <string>
#include <abmt/util/pimpl.h>
#include <abmt/io/pin.h>

namespace io{

// Good howto for chardev gpio: https://blog.lxsang.me/post/id/33

//@node: auto
struct gpio_output{
    std::string param_device = "/dev/gpiochip0";
    unsigned int param_pin = 0;
    
    bool in;
    
    abmt::util::pimpl data;
    
    void init();
    void tick();
    void final();
    
    ~gpio_output();
};

//@node: auto
struct gpio_input{
    std::string param_device = "/dev/gpiochip0";
    unsigned int param_pin = 0;
    
    bool out;
    
    abmt::util::pimpl data;
    
    void init();
    void tick();
    void final();
    
    ~gpio_input();
};

//@node: auto
//@raster: auto
struct gpio_soft_pwm{
    double in;
    bool out;
    
    size_t param_steps = 10;

    
    void tick();
};

struct linux_ipin: public abmt::io::ipin{
    gpio_input in;
    linux_ipin(std::string d, unsigned int p){
        in.param_device = d;
        in.param_pin = p;
        in.init();
    }
    virtual bool get(){
        in.tick();
		return in.out;
	}
};

struct linux_opin: public abmt::io::opin{
    gpio_output out;
    
    linux_opin(std::string d, unsigned int p){
        out.param_device = d;
        out.param_pin = p;
        out.init();
    }
    
    virtual void set(bool val = true){
        out.in = val;
        out.tick();
	}
};

//@node: auto
struct opin{
    std::string param_device = "/dev/gpiochip0";
    unsigned int param_pin = 0;
    
    abmt::io::opin_ptr out_opin;
    void init(){
        out_opin.reset(new linux_opin(param_device, param_pin));
    }
};

//@node: auto
struct ipin{
    std::string param_device = "/dev/gpiochip0";
    unsigned int param_pin = 0;
    
    abmt::io::ipin_ptr out_ipin;
    void init(){
        out_ipin.reset(new linux_ipin(param_device, param_pin));
    }
};

} // namespace io

#endif // IO_NODES_GPIO_H
