#include "spi.h"

#include <abmt/os.h>
#include <cstdint>
#include <fcntl.h> 
#include <sys/ioctl.h>
#include <linux/spi/spidev.h>

using namespace std;
using namespace io;
using namespace abmt;


struct linux_spi: public abmt::io::spi{
    int fd;
    int current_mode = -1;
    uint32_t speed;
    std::string device;
    
    linux_spi(std::string d, int param_mode, int param_freqzency){
        device = d;
        die_if( (fd = open(device.c_str(), O_RDWR)) < 0, "spi: Unable to open " + device);
        
        speed = param_freqzency;
        die_if(ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed), "spi: Error setting speed to " + to_string(param_freqzency));
 
        uint8_t mode_to_set = 0;
        if(param_mode > 0 && param_mode < 4){
            mode_to_set = param_mode;
        }
        die_if(ioctl(fd, SPI_IOC_WR_MODE, &mode_to_set) < 0, "spi: Error setting mode");
	    
    }
    virtual uint8_t tranceive(uint8_t c){
	abmt::blob b;
	b.data = (char*) &c;
	b.size = 1;
	tranceive(b);
	return c;
    }


    virtual void tranceive(abmt::blob& data){
        struct spi_ioc_transfer spi_msg = {0};
        spi_msg.tx_buf = (unsigned long) data.data;
        spi_msg.rx_buf = (unsigned long) data.data;
        spi_msg.len = data.size;
        //spi_msg.delay_usecs = 0;
        spi_msg.speed_hz = speed;
        spi_msg.bits_per_word = 8;
        spi_msg.cs_change = false;
      
        die_if(ioctl(fd, SPI_IOC_MESSAGE(1), &spi_msg) < 0, "Error sending spi data");
    }

};

void spi::init(){
    out_spi.reset(new linux_spi(param_device, param_mode, param_frequency));
}
