#include "tcp.h"
#include <abmt/os.h>

using namespace std;
using namespace io;

void tcp_server::init(){
    server.reset(new abmt::io::tcp_server(lst,{param_target, param_port}));
    server->on_new_connection = [&](abmt::io::tcp_ptr p){
		p->on_close = [this](){
		    for (auto it = connections.begin() ; it != connections.end(); ) {
                if ( (*it)->connected == false ) {
                    it = connections.erase(it);
                } else {
                    ++it;
                }
            }
		};
		p->on_new_data = [this](abmt::blob& b)->size_t{
			out += b;
		    return b.size;
		};
		connections.push_back(p);
	};
}

void tcp_server::tick(){
    if(in_snd_to_all.size > 0){
        for(auto con:connections){
            if(con->connected){
                con->send(in_snd_to_all);
            }
        }
    }
    out.realloc(0);
    lst.wait(0); // calls on_new_data and other events
    bool all_data_send = true;
    do{
        all_data_send = true;
        for(auto con:connections){
            if(con->connected && con->out_data_que.empty() == false){
                all_data_send = false;
                break;
            }
        }
        if(all_data_send == false){
            // this should rearly happen and is a sign for a bad
            // or a to slow connection / receiver.
            lst.wait(10);
        }
    } while(all_data_send == false);
}



void tcp_client::init(){
    connect();
    reconnect_timer.on_expiraton = [this]{
        if(con->connected == false){
            connect();
        }
    };
}

void tcp_client::tick(){
    if(con->connected && in.size > 0){
        con->send(in.data, in.size);
    }
    out.realloc(0);
    lst.wait(0); // calls on_new_data and other events
    while(con->connected && con->out_data_que.empty() == false){
        // this should rearly happen and is a sign for a bad
        // or a to slow connection / receiver.
        lst.wait(10);
    };
}

void tcp_client::connect(){
    con = abmt::io::tcp::connect_to(lst,{param_target,param_port});
    con->on_new_data = [this](abmt::blob& b)->size_t{
		out += b;
		return b.size;
	};
}